@@ -159,7 +159,6 @@ public:
 		inline bool isActive(void) const
 			{return active_flag;};
 
-		virtual void period(long slice);
 		virtual void cdrlog(cdr *call);
 		virtual void errlog(errlevel_t level, const char *text);
 		virtual bool check(void);
@@ -201,7 +200,7 @@ public:
 	static void dump(FILE *fp, keynode *node, unsigned level);
 	static void snapshot(const char *uid);
 	static void dumpfile(const char *uid);
-	static void period(long slice);
+	static bool period(long slice);
 	static void result(const char *value);
 	static FILE *open(const char *uid = NULL, const char *cfgpath = NULL);
 	static void startup(void);
@@ -218,6 +217,9 @@ public:
 	inline static keynode *getEnviron(void)
 		{return getProtected("environ");};
 
+	inline keynode *getRoot(void)
+		{return &root;};
+
 	virtual void dump(FILE *fp);
 	virtual void confirm(const char *user);
 	void commit(const char *user);
