@@ -539,7 +539,6 @@ service::keynode *server::getRouting(const char *id)
 	assert(cfg != NULL);
 
 	linked_pointer<keynode> node;
-	keynode *routing;
 	const char *cp;
 
 	// never re-route in-dialing nodes...
@@ -942,7 +941,8 @@ invalid:
 		if(!stricmp(argv[0], "message")) {
 			if(argc != 3)
 				goto invalid;
-			messages::system(argv[1], argv[2]);
+			if(messages::system(argv[1], argv[2]) != SIP_OK)
+				process::reply("cannot message");
 			continue;
 		}
 
