@@ -56,6 +56,8 @@ public:
 	class __LOCAL mapped : public MappedRegistry
 	{
 	public:
+		void update(Socket::address& addr, int changed);
+		void update(void);
 		bool expire(Socket::address& addr);
 		bool refresh(Socket::address& addr, time_t expires, const char *target_contact);
 		unsigned setTargets(Socket::address& addr);
@@ -95,6 +97,7 @@ public:
 	class __LOCAL target : public LinkedObject 
 	{ 
 	public: 
+		typedef enum {READY, BUSY, AWAY, DND, OFFLINE, UNKNOWN} status_t;
 		// internal hidden address indexing object
 		class indexing : public LinkedObject
 		{
@@ -106,8 +109,8 @@ public:
 		struct sockaddr_internet address;
 		struct sockaddr_internet iface;
 		time_t created; 
+		status_t status;
 		volatile time_t expires;
-		enum {READY, BUSY, AWAY, DND} status;
 		char contact[MAX_URI_SIZE]; 
 
 		static void *operator new(size_t size);
@@ -280,17 +283,17 @@ private:
 
 		enum {DIRECTED, CIRCULAR, TERMINAL, REDIRECTED, DISTRIBUTED} mode;
 
-		enum {LOCAL, INCOMING, OUTGOING, REFER} type;
+		destination_t type;
 
 		call();
 
 		Timer timer;
 		state_t state;
-		struct sockaddr_internet iface;	// source interface...
 		char forward[MAX_USERID_SIZE];	// ref id for forwarding...
 		char divert[MAX_USERID_SIZE];	// used in forward management
 		char dialed[MAX_IDENT_SIZE];	// user or ip address...
 		char subject[MAX_URI_SIZE];		// call subject
+		char request[MAX_URI_SIZE];		// requesting identity for refer flip
 		rtpproxy *rtp;
 
 		void disarm(void);
@@ -347,7 +350,7 @@ private:
 	void snapshot(FILE *fp);
 	bool check(void);
 
-	static void divert(stack::call *cr, struct sockaddr_internet *addr, osip_message_t *msg);
+	static void divert(stack::call *cr, osip_message_t *msg);
 
 	unsigned threading, priority;
 	size_t stacksize;
@@ -492,15 +495,16 @@ private:
 	void cleanup(void);
 	void snapshot(FILE *fp);
 
-	static bool deliver(message *msg);
+	static int deliver(message *msg);
+	static int remote(const char *to, message *msg, const char *digest = NULL);
 
 public:
 	messages();
 
 	static void automatic(void);
 	static void update(const char *userid);
-	static bool publish(const char *to, const char *reply, const char *from, caddr_t body, size_t size, const char *msgtype);
-	static bool system(const char *to, const char *message);
+	static int publish(const char *to, const char *reply, const char *from, caddr_t body, size_t size, const char *msgtype, const char *digest = NULL);
+	static int system(const char *to, const char *message);
 };
 
 
@@ -519,7 +523,6 @@ private:
 	registry::mapped *reginfo;
 	MappedRegistry *accepted;
 	eXosip_event_t *sevent;
-	char authbuf[1024];
 	char buffer[MAX_URI_SIZE];	
 	char identbuf[MAX_USERID_SIZE + 12];
 	char identity[MAX_USERID_SIZE];
@@ -562,6 +565,7 @@ private:
 	void registration(void);
 	void validate(void);
 	void message(void);
+	void publish(void);
 	void reregister(const char *contact, time_t interval);
 	void deregister(void);
 	void challenge(void);
