@@ -886,7 +886,7 @@ char *stack::sipAddress(struct sockaddr_internet *addr, char *buf, const char *u
 	assert(size > 0);
 
 	char pbuf[10];
-	unsigned port;
+	unsigned port = 0;
 	bool ipv6 = false;
 	const char *defaddr = NULL;
 
@@ -1018,7 +1018,7 @@ set:
 	return ap;
 }
 
-void stack::divert(stack::call *call, struct sockaddr_internet *iface, osip_message_t *invite)
+void stack::divert(stack::call *call, osip_message_t *invite)
 {
 	char route[MAX_URI_SIZE];
 	char touri[MAX_URI_SIZE];
@@ -1026,28 +1026,25 @@ void stack::divert(stack::call *call, struct sockaddr_internet *iface, osip_mess
 	if(!call->diverting)
 		return;
 
+	uri::publish(call->request, route, call->divert, sizeof(route));
+
 	if(String::equal(call->diverting, "all")) {
-		stack::sipPublish(iface, route, call->divert, sizeof(route));
 		snprintf(touri, sizeof(touri), "<%s>;reason=unconditional", route);
 		osip_message_set_header(invite, "Diversion", touri);
 	}
 	else if(String::equal(call->diverting, "na")) {
-		stack::sipPublish(iface, route, call->divert, sizeof(route));
 		snprintf(touri, sizeof(touri), "<%s>;reason=no-answer", route);
 		osip_message_set_header(invite, "Diversion", touri);
 	}
 	else if(String::equal(call->diverting, "busy")) {
-		stack::sipPublish(iface, route, call->divert, sizeof(route));
 		snprintf(touri, sizeof(touri), "<%s>;reason=user-busy", route);
 		osip_message_set_header(invite, "Diversion", touri);
 	}
 	else if(String::equal(call->diverting, "dnd")) {
-		stack::sipPublish(iface, route, call->divert, sizeof(route));
 		snprintf(touri, sizeof(touri), "<%s>;reason=do-not-disturb", route);
 		osip_message_set_header(invite, "Diversion", touri);
 	}
 	else if(String::equal(call->diverting, "away")) {
-		stack::sipPublish(iface, route, call->divert, sizeof(route));
 		snprintf(touri, sizeof(touri), "<%s>;reason=away", route);
 		osip_message_set_header(invite, "Diversion", touri);
 	}
@@ -1095,7 +1092,7 @@ void stack::inviteRemote(stack::session *s, const char *uri_target, const char *
 		return;
 	}
 
-	divert(call, &call->iface, invite);
+	divert(call, invite);
 
 	osip_message_set_header(invite, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
 	osip_message_set_header(invite, ALLOW_EVENTS, "talk, hold, refer");
@@ -1109,7 +1106,6 @@ void stack::inviteRemote(stack::session *s, const char *uri_target, const char *
 		char *req = NULL;
 		osip_uri_to_str(invite->req_uri, &req); 
 		snprintf(authbuf, 1024, "%s:%s", invite->sip_method, req);
-		osip_free(req); 
 		process::uuid(nounce, sizeof(nounce), "auth");
 		digest::md5(once, nounce);
 		if(!stricmp(registry::getDigest(), "sha1"))
@@ -1133,9 +1129,10 @@ void stack::inviteRemote(stack::session *s, const char *uri_target, const char *
 			",response=\"%s\""
 			",nonce=\"%s\""
 			",algorithm=%s"
-			,s->reg->userid, registry::getRealm(), uri_target, *response, *once, registry::getDigest());
+			,s->reg->userid, registry::getRealm(), req, *response, *once, registry::getDigest());
 		osip_message_set_header(invite, AUTHORIZATION, authbuf);
 		delete[] authbuf;
+		osip_free(req); 
 	}
 
 	if(call->expires) {
@@ -1149,7 +1146,7 @@ void stack::inviteRemote(stack::session *s, const char *uri_target, const char *
 	cid = eXosip_call_send_initial_invite(invite);
 	if(cid > 0) {
 		snprintf(seqid, sizeof(seqid), "%08x-%d", s->sequence, s->cid);
-		stack::sipAddress(&call->iface, route, seqid, sizeof(route));
+		uri::publish(call->request, route, seqid, sizeof(route));
 		eXosip_call_set_reference(cid, route);
 		++count;
 	}
@@ -1205,7 +1202,11 @@ repeat:
 		goto failed;
 
 	target = server::getValue(fwd, forwarding);
-	if(!target || !*target)
+
+	if((!target || !*target) && (String::equal(forwarding, "away") || String::equal(forwarding, "dnd")))
+		target = server::getValue(fwd, "busy");
+
+	if(!target || !*target) 
 		goto failed;
 
 	cr->diverting = forwarding;	
@@ -1264,7 +1265,6 @@ void stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t d
 	stack::session *invited;
 	stack::call *call = s->parent;
 	linked_pointer<stack::segment> sp = call->segments.begin();
-	unsigned away_count = 0, dnd_count = 0, busy_count = 0;
 
 	time_t now;
 	osip_message_t *invite;
@@ -1287,25 +1287,6 @@ void stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t d
 		}
 		sp.next();
 	}
-
-	switch(rr->status) {
-	case MappedRegistry::IDLE:
-		break;
-	case MappedRegistry::BUSY:
-		if(!call->count && call->forwarding)
-			call->forwarding = "busy";
-		return;
-	case MappedRegistry::DND:
-		if(!call->count && call->forwarding)
-			call->forwarding = "dnd";
-		return;
-	case MappedRegistry::AWAY:
-		if(!call->count && call->forwarding)
-			call->forwarding = "away";
-		return;
-	default:
-		return;
-	}
 	
 	while(is(tp)) {
 		invited = NULL;
@@ -1313,18 +1294,9 @@ void stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t d
 			goto next;
 
 		switch(tp->status) {
+		case registry::target::BUSY:		// can still try invite...
 		case registry::target::READY:
 			break;
-
-		case registry::target::AWAY:
-			++away_count;
-			goto next;
-		case registry::target::DND:
-			++dnd_count;
-			goto next;
-		case registry::target::BUSY:
-			++busy_count;
-			goto next;
 		default:
 			goto next;
 		}
@@ -1370,7 +1342,7 @@ void stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t d
 			osip_message_set_to(invite, touri);
 		}
 
-		divert(call, &tp->iface, invite);
+		divert(call, invite);
 
 		osip_message_set_header(invite, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
 		osip_message_set_header(invite, ALLOW_EVENTS, "talk, hold, refer");
@@ -1438,13 +1410,21 @@ next:
 		tp.next();
 	}
 
-	if(!count && call->forwarding) {
-		if(busy_count)
-			call->forwarding = "busy";
-		else if(dnd_count)
-			call->forwarding = "dnd";
-		else if(away_count)
-			call->forwarding = "away";
+	if(call->count > 0 || call->forwarding == NULL)
+		return;
+
+	switch(rr->status) {
+	case MappedRegistry::BUSY:
+		call->forwarding = "busy";
+		return;
+	case MappedRegistry::OFFLINE:
+		call->forwarding = "gone";
+		return;
+	case MappedRegistry::DND:
+		call->forwarding = "dnd";
+		return;
+	case MappedRegistry::AWAY:
+		call->forwarding = "away";
 		return;
 	}
 }
