@@ -52,19 +52,132 @@ thread::thread() : DetachedThread(stack::sip.stacksize)
 	session = NULL;
 }
 
+void thread::publish(void)
+{
+	int error = SIP_BAD_REQUEST;
+	osip_body_t *mbody = NULL;
+	const char *tmp;
+	registry::target::status_t status = registry::target::UNKNOWN;
+	bool presence = false;
+	bool basic = true;
+
+	if(destination != LOCAL)
+		goto final;
+
+	if(!reginfo || reginfo->type != MappedRegistry::USER)
+		goto final;
+
+	if(!String::equal(reginfo->userid, identity))
+		goto final;
+
+	osip_message_get_body(sevent->request, 0, &mbody);
+	if(!mbody) {
+		error = SIP_OK;
+		goto final;
+	}
+
+	tmp = mbody->body;
+	while(*tmp)
+	{
+		if(!strnicmp(tmp, "<presence", 9))
+			presence = true; 
+		else if(!strnicmp(tmp, "</basic", 7))
+			basic = false; 
+		else if(!strnicmp(tmp, "</status>", 9))
+			break; 
+		else if(!strnicmp(tmp, "<basic>open", 11) && basic) {
+			status = registry::target::READY;
+			tmp += 10;
+		}
+		else if(!strnicmp(tmp, "<show>dnd", 9) && presence) {
+			status = registry::target::DND;
+			break;
+		} 
+		else if(!strnicmp(tmp, "<basic>closed", 13) && basic) {
+			status = registry::target::OFFLINE;
+			break;
+		}
+		else if(!strnicmp(tmp, "available", 9) && presence) {
+			status = registry::target::READY;
+			break;
+		}
+		else if(!strnicmp(tmp, "unavailable", 11) && presence) {
+			status = registry::target::OFFLINE;
+			break;
+		}
+		else if(!strnicmp(tmp, "on-the-phone", 12) && presence) {
+			status = registry::target::BUSY;
+			break;
+		}
+		else if(!strnicmp(tmp, "online", 6) && presence) {
+			tmp += 5;
+			status = registry::target::READY;
+		}
+		else if(!strnicmp(tmp, "busy", 4) && presence) {
+			status = registry::target::BUSY;
+			break;
+		}
+		else if(!strnicmp(tmp, "dnd", 3) && presence) {
+			status = registry::target::DND;
+			break;
+		}
+		else if(!strnicmp(tmp, "vacation", 8) && presence)
+		{
+			status = registry::target::AWAY;
+			break;
+		}
+		else if(!strnicmp(tmp, "away", 4) && presence)
+		{
+			status = registry::target::AWAY;
+			break;
+		}
+		++tmp;
+	}
+
+	reginfo->update(via_address, status);
+	error = SIP_OK;
+
+final:
+	send_reply(error);
+}
+
 void thread::message(void)
 {
 	osip_content_type_t *ct;
 	osip_body_t *body = NULL;
 	char address[MAX_URI_SIZE];
 	char fromhdr[MAX_URI_SIZE];
+	char target[MAX_URI_SIZE];
 	char msgtype[64];
 	char sysid[64];
 	char *msglen = NULL;
-	const char *id;
+	const char *id = NULL;
+	const char *digest = NULL;
 
-	if(!dialed.keys || destination != LOCAL) {
-		debug(3, "cannot send message for %s", dialing);
+	switch(destination) {
+	case LOCAL:
+		if(!dialed.keys) {
+			send_reply(SIP_DECLINE);
+			return;
+		}
+		id = service::getValue(dialed.keys, "extension");
+		if(!id)
+			id = service::getValue(dialed.keys, "id");
+		String::set(target, sizeof(target), id);
+		break;
+	case EXTERNAL:
+		String::set(target, sizeof(target), requesting);
+		break;
+	case REDIRECTED:
+		if(authorized.keys)
+			digest = server::getValue(authorized.keys, "digest");
+		if(!digest) {
+			send_reply(SIP_DECLINE);
+			return;
+		}
+		String::set(target, sizeof(target), requesting);	
+		break;
+	default:
 		send_reply(SIP_DECLINE);
 		return;
 	}
@@ -83,12 +196,12 @@ void thread::message(void)
 		snprintf(msgtype, sizeof(msgtype), "%s",
 			ct->type);
 
-	if(extension)
+	if(extension && !digest)
 		snprintf(sysid, sizeof(sysid), "%u", extension);
 	else
 		String::set(sysid, sizeof(sysid), identity);
 
-	stack::sipPublish(&iface, address, sysid, sizeof(address));
+	uri::publish(requesting, address, sysid, sizeof(address));
 	if(extension && !display[0])
 		snprintf(fromhdr, sizeof(fromhdr), 
 			"\"%u\" <%s;user=phone>", extension, address);
@@ -99,21 +212,13 @@ void thread::message(void)
 		snprintf(fromhdr, sizeof(fromhdr),
 			"<%s>", address);
 
-
-	id = service::getValue(dialed.keys, "extension");
-	if(!id)
-		id = service::getValue(dialed.keys, "id");
-
-	debug(3, "sending message from %s to %s\n", sysid, id);
+	debug(3, "sending message from %s to %s\n", sysid, target);
 	osip_content_length_to_str(sevent->request->content_length, &msglen);
 	if(!msglen) {
 		send_reply(SIP_BAD_REQUEST);
 		return;
 	}
-	if(messages::publish(id, sysid, fromhdr, body->body, atoi(msglen), msgtype))		
-		send_reply(SIP_OK);
-	else
-		send_reply(SIP_MESSAGE_TOO_LARGE);
+	send_reply(messages::publish(target, sysid, fromhdr, body->body, atoi(msglen), msgtype, digest));
 	osip_free(msglen);
 }
 
@@ -127,7 +232,7 @@ void thread::invite(void)
 	char fromext[32];
 	cdr *cdrnode;
 
-	memcpy(&session->parent->iface, &iface, sizeof(iface));
+	uri::serviceid(requesting, call->request, sizeof(call->request));
 
 	// FIXME: we should get proxy count extimate from sdp into global thread object...
 
@@ -190,10 +295,10 @@ noproxy:
 	cdrnode->sequence = session->sequence;
 	cdrnode->cid = session->cid;
 	String::set(cdrnode->uuid, sizeof(cdrnode->uuid), session->uuid);	
-
+	
+	call->type = destination;
 	switch(destination) {
 	case LOCAL:
-		call->type = stack::call::LOCAL;
 		if(extension)
 			snprintf(session->sysident, sizeof(session->sysident), "%u", extension);
 		else
@@ -242,7 +347,6 @@ noproxy:
 			session->sequence, session->cid, call->dialed, session->sysident);
 		break;
 	case PUBLIC:
-		call->type = stack::call::INCOMING;
 		String::set(call->dialed, sizeof(call->dialed), target);
 		snprintf(session->identity, sizeof(session->identity), "%s:%s@%s:%s",
 			from->url->scheme, from->url->username, from->url->host, from->url->port);
@@ -270,7 +374,6 @@ noproxy:
 		break;
 	case REDIRECTED:
 	case EXTERNAL:
-		call->type = stack::call::OUTGOING;
 		if(extension)
 			snprintf(session->sysident, sizeof(session->sysident), "%u", extension);
 		else
@@ -309,7 +412,6 @@ noproxy:
 		session->closed = false;
 		goto exit;
 	case ROUTED:
-		call->type = stack::call::OUTGOING;
 		call->phone = true;
 
 		if(extension)
@@ -359,8 +461,8 @@ noproxy:
 		// get rid of config ref if we are calling registry target
 		server::release(dialed);
 
-		String::set(session->parent->forward, MAX_USERID_SIZE, reginfo->userid);
-		session->parent->forwarding = "na";
+		String::set(call->forward, MAX_USERID_SIZE, reginfo->userid);
+		call->forwarding = "na";
 		stack::inviteLocal(session, reginfo, destination);
 	}
 
@@ -369,7 +471,7 @@ noproxy:
 	}
 
 exit:
-	if(!call->invited && !stack::forward(session->parent)) {
+	if(!call->invited && !stack::forward(call)) {
 		call->busy(this);
 		return;
 	}
@@ -1812,6 +1914,10 @@ void thread::run(void)
 					message();
 				break;
 			}
+			else if(MSG_IS_PUBLISH(sevent->request)) {
+				if(authorize())
+					publish();
+			}
 			else if(!MSG_IS_INFO(sevent->request)) {
 				debug(2, "unsupported %s in dialog", sevent->request->sip_method);
 				break;
